/*
 * Copyright 2005, 2006 University of Leiden.
 *
 * This file is part of MIA+EWS.
 *
 * MIA+EWS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA+EWS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <math.h>
#include "oirSupport.h"
#include "fitsio.h"
#include "imaging_data.h"
#include "imaging_fdata.h"
#include "minrtsFrameData.h"
#include "minrtsFftUtil.h"

#define yBottom 5
#define yTop    34
#define NBIN 5
#define DEBUG 1
#define SIGMAY 2.0
#define RECAL 1
/**********************************************************
 * Usage:
 *    oirSciPhotometry -data "data" -ref coordFile -cross crosscoeff \
 *       -out outFile -mask outMaskFile -sky "skyFiles" \ 
 *       -ndrop nDrop -sigma sigmaY
 *
 *    data is a blank-separated list of input files
 *
 *    coordFile is a fits file containing at least a detector def file
 *      defining the wavelength/distortion coefficients
 *
 *    crosscoeff is a crosscoefficient (kappa coefficient) file name
 *
 *    outFile is the name of the output file
 *
 *    outMaskFile contains the name of a file to contain the
 *      photometric mask (to be used in fringe estimation program)
 *
 *    
 *    "skyFiles" is optional to be used if the main SCI_PHOT
 *    files were not taken in Chopped mode.  The skyfiles are
 *    one or more files in SCI_PHOT/chop mode with the same
 *    exposure times, from which sky images of both PA and
 *    PB channels can be determined.
 *    
 *    sigmaY  sigma of masks in Y-direction (pixels) (optional)
 ************************************************************* */

void Usage() {
   printf("Usage: oirSciPhotometry -data ""inFiles"" \n");
   printf("    [-mask maskFile]\n");
   printf("    -out outFile\n");
   printf("    -ref refCoordFile\n");
   printf("    -cross crossCoeffFile\n");
   printf("    -sky  ""skyFiles"" \n");
   printf("    -sigma sigmaY\n");
   printf("inFiles: a blank-separated list of input files\n");
   printf("enclosed in double quotes if indeed blank-separated\n");
   printf("MaskFile: the mask generated by oirSciPhotCal\n");
   printf("outFile: output file name, input to oirFormFringes\n");
   printf("refCoordFile: contains coordinate distortion info:\n");
   printf("sigmaY: sigma (std. dev.) of gaussian mask in pixels\n");
   printf("defaults to 2.0\n");
}
/* ************************************************************************
   *  Create a pixel weighting mask for each window as a gaussian
   *  whose y-center is found by summing the photometric channels in
   *  the x-direction, finding the peak, and taking the mean of
   *  the PA and PB peaks.  The sigma width of the gaussian is
   *  specified externally.
   *   Inputs:
   *  \param  Target  *minrtsFrameData  structure containing sky-subtracted
   *                                    photometric images
   *  \param  sigmaY  float   specified standard deviation width of gaussian in y
   ***********************************************************************   */
minrtsFrameData   *midiCreateMask(minrtsFrameData *Target, float sigmaY) {

   minrtsFrameData *mask;
   int iy, ix, nx, ny ;
   int i,j, ierr;;
   int iregion;

   float *sumX, *sumY, *yy;
   float y, yC[4], maxx;

   nx = Target->nx[0];
   ny = Target->ny[0];

   sumX = (float *)malloc(ny*sizeof(float));
   sumY = (float *)malloc(nx*sizeof(float));
   yy   = (float *)malloc(nx*sizeof(float));
/*
      loop over PA and PB
      first sum over (most of) Y to find limits in X
      them sum over X to find peak row
*/
   for (i=0;i<ny;i++) yy[i] = (float)i;
   for (i=0; i<4; i=i+3) {
       for (ix=0;ix<nx;ix++) {
          sumY[ix] = 0.;
          for (iy=5; iy<ny-5; iy++) sumY[ix] = sumY[ix] + 
             Target->data[i][iy*nx+ix];
       }
       maxx = -1.e9;
       for (ix=0;ix<nx;ix++) if (sumY[ix] > maxx) maxx = sumY[ix];
       maxx = 0.1 * maxx;
       for (iy=0; iy<ny; iy++) {
          sumX[iy] = 0.;
          for (ix=0;ix<nx;ix++) if (sumY[ix] > maxx) sumX[iy] = sumX[iy] + 
             Target->data[i][iy*nx+ix];
       }
       y = minrtsQPk(ny, yy, sumX, &maxx, 0);
       yC[i] = y;
   }
/*
      average peak of channels 1 and 4
*/
   yC[0] = 0.5*(yC[0]+yC[3]);
      
/*
      create empty object
*/
   mask = minrtsFrameDataClone(Target);
/*
      create gaussian masks that follow curve in coordinate system
*/
   sigmaY = 0.5/(sigmaY*sigmaY);
   for (iregion=0; iregion<4; iregion++) {
      for (ix=0;ix<nx;ix++) {
         for (iy=0;iy<ny;iy++) {
            maxx = sigmaY * (iy-yC[0])*(iy-yC[0]);
            if (maxx < 5.) maxx = exp(-maxx); else maxx = 0.;
            mask->data[iregion][ix+nx*iy] = maxx;
         }
      }
   }
   free(sumX);
   free(sumY);
   free(yy);
   return mask;
} /* end of midiCreateMask */


/* *********************************************************
   *  use top and bottom rows to estimate sky in each
   *  window and subtract linear fit
   ********************************************************** */

void deSky (minrtsFrameData *data, int y1, int y2) {
   int ix, iy;
   float dy, x1, x2;
   float topSky, bottomSky;
   float *yLine;
   int nx;
   int iregion;

   for (iregion=0; iregion<data->nregion; iregion++) {
      nx = data->nx[iregion];

/*
      average a few rows and do a linear fit
*/
      dy = y2 - y1;
      for (ix=0; ix<nx; ix++) {
         for (iy =-1; iy<=1; iy++) {
            topSky    = topSky +    data->data[iregion]
               [ix + (iy+y2)*(long)nx];
            bottomSky = bottomSky + data->data[iregion]
               [ix + (iy+y1)*(long)nx];
         }
         topSky    = topSky/3.;
         bottomSky = bottomSky/3.;
/*
      linear fit and subtraction
*/
         for (iy=0; iy<data->ny[iregion]; iy++) {
            x1 = ((float)iy - y1)/dy;
            x2 = ((float)iy - y2)/dy;
            data->data[iregion][ix+iy*nx] = data->data[iregion][ix+iy*nx]-
               topSky*x1 + bottomSky*x2;
         } /* loop over y */
      }    /* loop over x */
   } /* loop over regions */
}

/* ************************************************************************
   *  If the sky background frames are not in the original data because
   *  it wasn't chopped, we can extract them from data files taken
   *  with the same setup, but chopped (usually PHOTOMETRY exposures
   *  taken just before/after INTERFEROMETRY exposures).  These 
   *  exposures must of course have some sky frames for both the
   *  A- and B- photometric channels.
   *   Inputs:
   *  \param  skyFileList  **char   list of input files
   *  \param  nFiles       int      how many of above
   *  \param  Sky          *minrtsFrameData  output structure containing
   *                                         averaged sky frames
   *  \param  nDrop        int      frames to drop after chop transition
   *  \param  ierr        *int      0 is OK.
   ********************************************************************* */
void getSkyFrames(char **skyFileList, int nFiles, minrtsFrameData *Sky, 
   int nDrop, int *ierr) {

   int i, iFile, ix, iy;
   int nx, ny;
   float nA, nB;

   minrtsFrameData *tTarg, *tSky;

   fitsfile *inFile;

   char  shutter[12];
   char  opt1[12];

   char *AShutter = "AOPEN";
   char *BShutter = "BOPEN";
   char *ABShutter = "ABOPEN";
   char *sciphot = "SCI_PHOT";

   *ierr = 0;
   nA = 0.;
   nB = 0.;

   nx = Sky->nx[0];
   ny = Sky->ny[0];

  
   for (i=0;i<(nx*ny);i++) {
      Sky->data[0][i] = 0.;
      Sky->data[3][i] = 0.;
   }
/*
      create two temporary minrtsFrameDatas
*/
   tTarg = minrtsFrameDataClone(Sky);
   tSky  = minrtsFrameDataClone(Sky);
/*
      loop over input files, using unChop to extract sky
      into temp file, also check which shutters were open
      for each file and that we are in SCI_PHOT mode
*/
   *ierr = 0;
   for (iFile=0; iFile<nFiles; iFile++) {
      i = fits_open_file(&inFile, skyFileList[iFile], READONLY, ierr) ;
      if (*ierr != 0) {
         printf("ERROR:cfitsio %i opening sky file %s\n", ierr, skyFileList[iFile]);
         printf("%s \n",strerror(errno));
         return;
      }
      i = fits_read_key(inFile, TSTRING, "HIERARCH ESO INS SHUT NAME", shutter,
         0, ierr);
      if (*ierr != 0) {
         printf("couldnt read shutter from %s\n", skyFileList[iFile]);
         return;
      }
      i = fits_read_key(inFile, TSTRING, "HIERARCH ESO INS OPT1 NAME", opt1,
         0, ierr);
      if (*ierr != 0) {
         printf("couldnt read opt1 from %s\n", skyFileList[iFile]);
         return;
      }
      if (strcmp(opt1, sciphot) != 0) {
         printf("data from %s not in SCI_PHOT mode\n", skyFileList[iFile]);
         *ierr = 1;
         return;
      }
      i = fits_close_file(inFile, ierr);
      minrtsFrameDataUnChop(skyFileList+iFile, 1, nDrop,
         tTarg, tSky, ierr);
      if (*ierr != 0) {
         printf("Sky extraction failed on %s \n",skyFileList[iFile]);
         return;
      }
      if ((tSky->nx[0] != nx) || (tSky->ny[0] != ny)) {
         printf("window size of %s; %i %i not compatible with data: %i %i\n ",
            skyFileList[iFile], tSky->nx[0], tSky->ny[0], nx, ny);
            *ierr = 1;
         return;
      }
      if (tSky->weight > 0.) {
/*
             check shutter status and accumulate data -> Sky object
*/
         if ((strcmp(shutter, AShutter) ==0) || (strcmp(shutter, ABShutter) ==0)) {
            for (i=0;i<(nx*ny);i++) Sky->data[3][i] = Sky->data[3][i]+tSky->data[3][i]*tSky->weight;
            nA = nA + tSky->weight;
         }
         if ((strcmp(shutter, BShutter) ==0) || (strcmp(shutter, ABShutter) ==0)) {
            for (i=0;i<(nx*ny);i++) Sky->data[0][i] = Sky->data[0][i]+tSky->data[0][i]*tSky->weight;
            nB = nB + tSky->weight;
         }
      }
   }
   if ((nA <1.) || (nB < 1.)) {
      printf("sky files did not contain enough sky frames nA=%7.0f nB=%7.0f\n", nA, nB);
      *ierr = 1;
   }
   printf("In sky files found sky frames nA=%7.0f nB=%7.0f\n", nA, nB);
   for (i=0;i<(nx*ny);i++) Sky->data[0][i] = Sky->data[0][i]+tSky->data[0][i]/nB;
   for (i=0;i<(nx*ny);i++) Sky->data[3][i] = Sky->data[3][i]+tSky->data[3][i]/nA;
   Sky->weight = 0.5*(nA + nB);

   minrtsFrameDataKill(tSky);
   minrtsFrameDataKill(tTarg);
   return;
} /* end of getSkyFrames */

int main (int argc, char** argv) {

   char opt1[12];
   char gris[12];
   char shutter[12];
   float dit;

   char **fileList;
   char **skyFileList;

   int iFile, nFiles;
   int iShutter;
   int i,ierr;
   int nx, ny;
   long npix;
   long iPix;
   int  ix, iy;
   float xx, mm;
   long  ii;
   int iKey;
   int nCycle;

   int chan1, chan2;

   char outFileName[130];
   char maskFileName[130];

   fitsfile *inFile;

   imaging_fdata_table_def *odata;
   imaging_detector_table_def *DetectorDef;
   imaging_detector_table_def *CoordDef;
   imaging_fdata_row *outData;
   int nRegion;


   minrtsFrameData *Target;
   minrtsFrameData *Sky;
   minrtsFrameData *Mask;
   minrtsFrameData *Phot;
   minrtsFrameData *Cross ;

   int iarg;

   int   nDrop ;
   float sigmaY ; 

   float *ATotal, *BTotal, *ABTotal;
   float *AMask, *BMask, *ABMask;

   float t1, t2;
   float p02, p13;

   char *bang="!";
   char *ABshutter   = "ABOPEN";
   char *sciPhot     = "SCI_PHOT";
   fitsfile *outFile = NULL;
   int one = 1;

   char *inFileName    = NULL;
   char *skyFileName   = NULL;
   char *refFileName   = NULL;
   char *crossFileName = NULL;

   int  *nFrames;

   ierr = 0;
/*
      parse input
*/
   sigmaY = SIGMAY;
   nDrop  = 0;
   iarg   = 0;
   maskFileName[0] = 0;
   outFileName[0]  = 0;
   while (iarg < argc) {
      if (strcmp(argv[iarg], "-data")==0) 
        inFileName = argv[++iarg];
      if (strcmp(argv[iarg], "-sky")==0) 
        skyFileName = argv[++iarg];
      if (strcmp(argv[iarg], "-cross")==0) 
        crossFileName = argv[++iarg];
      if (strcmp(argv[iarg], "-mask")==0) {
         strcpy(maskFileName, bang);
         strcat(maskFileName, argv[++iarg]);
      }
      if (strcmp(argv[iarg], "-ndrop")==0) 
        sscanf(argv[++iarg],"%i",&nDrop);
      if (strcmp(argv[iarg], "-sigma")==0) 
        sscanf(argv[++iarg],"%f",&sigmaY);
      if (strcmp(argv[iarg], "-out")==0) {
         strcpy(outFileName, bang);
         strcat(outFileName, argv[++iarg]);
      }
      if (strcmp(argv[iarg], "-ref")==0) 
        refFileName = argv[++iarg];
      ++iarg;
   }
   if (inFileName == NULL) {
      printf("ERROR:you need to specify some input files \n");
      return 1;
   }
   if (refFileName == NULL) {
      printf("ERROR:you need to specify the coordinate reference file name\n");
      return 1;
   }
   if (crossFileName == NULL) {
      printf("ERROR:you need to specify the cross coefficient file name\n");
      return 1;
   }
   if (outFileName[0] == 0) {
      printf("ERROR:you need to specify the output file name\n");
      return 1;
   }
   if (maskFileName[0] == 0) {
      printf("ERROR:you need to specify the output mask file name\n");
      return 1;
   }
if (skyFileName != NULL) printf("skyfilename %s\n", skyFileName);
/*        
       break up file list and print what you find 
       then read and unchop files
*/
   fileList = oirStrSplit(inFileName, &nFiles);

/*
       go through and check that all files are there
       and get nFrames/file
*/
   nFrames = (int *)malloc(nFiles * sizeof(int));
   ierr = 0;
   for (iFile=0;iFile<nFiles; iFile++) {
      i = fits_open_file(&inFile, fileList[iFile], READONLY, &ierr) ;
      if (ierr != 0) {
         printf("ERROR:cfitsio error %i opening file %s\n", ierr, fileList[iFile]);
         printf("%s \n",strerror(errno));
         return 1;
      }
      i = fits_read_key(inFile, TINT, "HIERARCH ESO DET NDIT", nFrames+iFile, 0, &ierr);
      printf("   %s contains %i frames \n", fileList[iFile], nFrames[iFile]);
      i = fits_read_key(inFile, TSTRING, "HIERARCH ESO INS SHUT NAME", 
         shutter, 0, &ierr);
      if (strncmp(shutter, ABshutter, 6)!=0) {
         printf("ERROR:I expect shutter ABOPEN not %s \n", shutter);
         return 1;
      }
      i = fits_read_key(inFile, TFLOAT, "HIERARCH ESO DET DIT", &dit, 
         0, &ierr);
      i = fits_read_key(inFile, TSTRING, "HIERARCH ESO INS OPT1 NAME", 
         opt1,0, &ierr);
      if (strcmp(opt1, sciPhot) != 0) {
         printf("ERROR:I expect opt1 to be SCI_PHOT not %s \n", opt1);
         return 1;
      }
      i = fits_read_key(inFile, TSTRING, "HIERARCH ESO INS GRIS NAME", 
         gris,0,&ierr);
/*
      move to detector section
*/
      if (iFile == 0) {
         DetectorDef = read_imaging_detector_table(inFile, &ierr);
         if (ierr !=0) {
            printf("ERROR:%i opening detector table in %s\n", 
               ierr, fileList[iFile]);
            return 1;
         }
         nRegion = DetectorDef->nregion;
      } /* detector from first file */
      i = fits_close_file(inFile, &ierr);
   }   /* look at input files */
   printf("   nDrop = %i\n", nDrop);
   printf("   sigmaY = %f\n", sigmaY);
   printf("   dit = %f\n", dit);
   printf("   disperser is = %s\n", gris);

/*
       get detector table from file with coordinate distortion info
*/

   i = fits_open_file(&inFile, refFileName, READONLY, &ierr) ;
   if (ierr != 0) {
      printf("ERROR:cfitsio %i opening coord file %s\n", ierr, refFileName);
      printf("%s \n",strerror(errno));
      return 1;
   }
   CoordDef = read_imaging_detector_table(inFile, &ierr);
   if (ierr !=0) {
      printf("ERROR: %i opening Coord detector table in %s\n", 
         ierr, refFileName);
      return 1;
   }
   printf("   using optical distortion data from %s\n", refFileName);
   i = fits_close_file(inFile, &ierr);

/*
          create FrameData structures to hold data
*/
   Target = minrtsFrameDataMake(DetectorDef);
   Sky    = minrtsFrameDataMake(DetectorDef);
   Mask   = minrtsFrameDataMake(DetectorDef);
   Phot   = minrtsFrameDataMake(DetectorDef);
/*
          now read in the real data using UnChop utility
*/
   
   printf("   Averaging Target and Sky Frames\n");
   minrtsFrameDataUnChop (fileList, nFiles, nDrop,
      Target, Sky, &ierr);
   if (ierr != 0) {
      printf("ERROR:Failed to process Data: error: %i\n", ierr);
      printf("%s \n", strerror(errno));
      return 1;
   }
   printf("   Found %7.0f Target and %7.0f Sky frames in Interferometric files\n", 
      Target->weight, Sky->weight);
   oirKillFileList(fileList, nFiles);
/*
          do I need to get Sky from another file
*/
   if (skyFileName != NULL) {
      printf("Processing sky files\n");
      skyFileList = oirStrSplit(skyFileName, &nFiles);
      getSkyFrames(skyFileList, nFiles, Sky, nDrop, &ierr);
      if (ierr != 0) {
         printf("ERROR:Failed to process sky Data: error: %i\n", ierr);
         printf("%s \n", strerror(errno));
         return 1;
   }
      oirKillFileList(skyFileList, nFiles);
   }

   if ((Target->weight < 0.5) || (Sky->weight < 0.5)) {
      printf("ERROR:Need nonZero Target and Sky Frames to proceed\n");
      return 1;
   }
#ifdef DEBUG 
minrtsFrameDataWriteFits(Target, "target.fits", &ierr);
minrtsFrameDataWriteFits(Sky, "sky.fits", &ierr);
#endif
/*
       subtract sky and divide by dit
*/
   minrtsFrameDataSub(Target, Target, Sky);

#ifdef DEBUG 
minrtsFrameDataWriteFits(Target, "diff.fits", &ierr);
#endif

   if (dit > 0.) minrtsFrameDataScalarMul(Target, 1./dit);

/*
      deSky and Rectify images
*/
   deSky(Target, yBottom, yTop);
#ifdef DEBUG 
minrtsFrameDataWriteFits(Target, "desky.fits", &ierr);
#endif
   midiRectXYWindow(Target, CoordDef, 2, &ierr);
#ifdef DEBUG 
minrtsFrameDataWriteFits(Target, "rect.fits", &ierr);
#endif
   nx = Target->nx[0];
   ny = Target->ny[0];
/*
      allocate output arrays
*/
   ATotal = (float *)malloc(nx* sizeof(float));
   BTotal = (float *)malloc(nx* sizeof(float));
   ABTotal= (float *)malloc(nx* sizeof(float));
   AMask  = (float *)malloc(nx* sizeof(float));
   BMask  = (float *)malloc(nx* sizeof(float));
   ABMask = (float *)malloc(nx* sizeof(float));
/*
      duplicate A-B photometry before applying cross coupling coeff
*/
   for (ix=0;ix<nx;ix++) for (iy=0;iy<ny;iy++) {
      i = ix + iy*nx;
      Phot->data[0][i] = Target->data[3][i];
      Phot->data[1][i] = Target->data[3][i];
      Phot->data[2][i] = Target->data[0][i];
      Phot->data[3][i] = Target->data[0][i];
   }
/*
      create mask arrays and save
*/
   Mask = midiCreateMask(Target, sigmaY);
   minrtsFrameDataWriteFits(Mask, maskFileName, &ierr);
   if (ierr != 0) {
      printf("ERROR: %i writing mask data to file %s\n", ierr, maskFileName);
      printf("%s \n", strerror(errno));
      return 1;
   }
   printf("   writing mask file into %s\n", maskFileName);
/*
      get cross coupling coefficients
      multiply Phot by Cross
*/
   Cross = minrtsFrameDataReadFitsRow(crossFileName,1,&ierr);
   if (ierr != 0) {
      printf("ERROR: %i getting kappa coeff from file %s\n", ierr, crossFileName);
      printf("%s \n", strerror(errno));
      return 1;
   }
   printf("   getting kappa coefficients from  %s\n", crossFileName);
   for (i=0;i<4;i++) for(ix=0;ix<nx;ix++) for (iy=0;iy<ny;iy++) 
         Phot->data[i][ix+iy*nx] = Phot->data[i][ix+iy*nx]*Cross->data[i][ix];
#ifdef DEBUG 
minrtsFrameDataWriteFits(Phot, "Phot.fits", &ierr);
#endif
#ifdef RECAL
/*
      modify effective kappa coeff to make sum of Phot[0+2] = Target[1]
      modify effective kappa coeff to make sum of Phot[1+3] = Target[3]
*/
   for (ix=0;ix<nx;ix++) {
      p02 = 0.;
      t1  = 0.;
      p13 = 0.;
      t2  = 0.;
      for (iy=yBottom;iy<yTop;iy++) {
         i = ix + iy * nx;
         p02 = p02+Phot->data[0][i] + Phot->data[2][i];
         p13 = p13+Phot->data[1][i] + Phot->data[3][i];
         t1  = t1 +Target->data[1][i];
         t2  = t2 +Target->data[2][i];
      }
      if (p02 > 0.) t1 = t1/p02; else t1 = 1.;
      if (p13 > 0.) t2 = t2/p13;else t2 = 1.;
      for (iy=0;iy<ny;iy++) {
         i = ix + iy * nx;
         Phot->data[0][i] = Phot->data[0][i] * t1;
         Phot->data[2][i] = Phot->data[2][i] * t1;
         Phot->data[1][i] = Phot->data[1][i] * t2;
         Phot->data[3][i] = Phot->data[3][i] * t2;
      }
   }
#ifdef DEBUG 
minrtsFrameDataWriteFits(Phot, "PhotC.fits", &ierr);
#endif
#endif
/*
      compute  totals
*/
   for (ix=0;ix<nx;ix++) {
      ATotal[ix]  = 0.;
      BTotal[ix]  = 0.;
      ABTotal[ix] = 0.;
      AMask[ix]  = 0.;
      BMask[ix]  = 0.;
      ABMask[ix] = 0.;
      for (iy=yBottom;iy<yTop;iy++) {
         i = ix + iy*nx;
         xx = Phot->data[0][i] + Phot->data[1][i];
         ATotal[ix] = ATotal[ix] + xx;
         AMask[ix]  = AMask[ix] + xx * Mask->data[1][i];
         xx = Phot->data[2][i] + Phot->data[3][i];
         BTotal[ix] = BTotal[ix] + xx;
         BMask[ix]  = BMask[ix] + xx * Mask->data[1][i];
         xx = Phot->data[0][i]*Phot->data[2][i];
         if (xx > 0.) {
            xx = sqrt(xx);
            ABTotal[ix] = ABTotal[ix] + xx;
            ABMask[ix] = ABMask[ix] + xx * Mask->data[1][i];
         }
         xx = Phot->data[1][i]*Phot->data[3][i];
         if (xx > 0.) {
            xx = sqrt(xx);
            ABTotal[ix] = ABTotal[ix] + xx;
            ABMask[ix] = ABMask[ix] + xx * Mask->data[1][i];
         }
      }  /* sum over y */
   } /* loop over x */
/*
      prepare output file
*/

   fits_create_file (&outFile, outFileName, &ierr);
   if (ierr !=0) {
      printf ("ERROR: opening fits output %s %i\n", outFileName, ierr);
      printf("%s \n",strerror(errno));
      return 1;
   }
   printf("   writing output photometry into %s\n",outFileName);
   DetectorDef->nregion = 1;
   DetectorDef->naxis[0][1] = 1;
   iKey = table_find_key(DetectorDef->def, "NREGION");
   table_put_keyword(DetectorDef->def, iKey, (void *)&one, &ierr);

   odata = make_imaging_fdata_table_def (outFile, DetectorDef, 1, &ierr);
   if (ierr !=0) {
      printf ("ERROR:creating data definition\n");
      return 1;
   }
   outData = make_imaging_fdata_row(odata);

   for (ix=0;ix<nx;ix++) outData->data[0][ix] = ATotal[ix];
   i = write_imaging_fdata_table(outFile, odata, outData, 1, &ierr);

   for (ix=0;ix<nx;ix++) outData->data[0][ix] = BTotal[ix];
   i = write_imaging_fdata_table(outFile, odata, outData, 2, &ierr);

   for (ix=0;ix<nx;ix++) outData->data[0][ix] = ABTotal[ix];
   i = write_imaging_fdata_table(outFile, odata, outData, 3, &ierr);

   for (ix=0;ix<nx;ix++) outData->data[0][ix] = AMask[ix];
   i = write_imaging_fdata_table(outFile, odata, outData, 4, &ierr);

   for (ix=0;ix<nx;ix++) outData->data[0][ix] = BMask[ix];
   i = write_imaging_fdata_table(outFile, odata, outData, 5, &ierr);

   for (ix=0;ix<nx;ix++) outData->data[0][ix] = ABMask[ix];
   i = write_imaging_fdata_table(outFile, odata, outData, 6, &ierr);

   for (ix=0;ix<nx;ix++) outData->data[0][ix] = 0.;/* ATotal[ix];*/
   i = write_imaging_fdata_table(outFile, odata, outData, 7, &ierr);

   for (ix=0;ix<nx;ix++) outData->data[0][ix] = 0.;/*BTotal[ix];*/
   i = write_imaging_fdata_table(outFile, odata, outData, 8, &ierr);

   for (ix=0;ix<nx;ix++) outData->data[0][ix] = 0.;/*ABTotal[ix];*/
   i = write_imaging_fdata_table(outFile, odata, outData, 9, &ierr);

   for (ix=0;ix<nx;ix++) outData->data[0][ix] = 0.;/*AMask[ix];*/
   i = write_imaging_fdata_table(outFile, odata, outData, 10, &ierr);

   for (ix=0;ix<nx;ix++) outData->data[0][ix] = 0.;/*BMask[ix];*/
   i = write_imaging_fdata_table(outFile, odata, outData, 11, &ierr);

   for (ix=0;ix<nx;ix++) outData->data[0][ix] = 0.;/*ABMask[ix];*/
   i = write_imaging_fdata_table(outFile, odata, outData, 12, &ierr);

   i = close_imaging_fdata_table(outFile, odata, &ierr);
   kill_imaging_fdata_row(outData);

   minrtsFrameDataKill(Target);
   minrtsFrameDataKill(Sky);
   minrtsFrameDataKill(Mask);
   minrtsFrameDataKill(Phot);
   minrtsFrameDataKill(Cross);
   free(AMask);
   free(BMask);
   free(ABMask);
   free(ATotal);
   free(BTotal);
   free(ABTotal);
   free(nFrames);
   return 0;
}
